@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@
 #
 
 __author__    = 'Thomas Perl <thp@gpodder.org>'
-__version__   = '2.1'
-__date__      = '2009-12-12'
-__copyright__ = '© 2005-2009 Thomas Perl and the gPodder Team'
+__version__   = '2.2'
+__date__      = '2010-02-05'
+__copyright__ = '© 2005-2010 Thomas Perl and the gPodder Team'
 __licence__   = 'GNU General Public License, version 3 or later'
 __url__       = 'http://gpodder.org/'
 
@@ -41,6 +41,15 @@ except ImportError:
     sys.exit(1)
 del feedparser
 
+try:
+    import mygpoclient
+except ImportError:
+    print """
+  Error: Module "mygpoclient" not found. Please install "python-mygpoclient"
+         or download it from http://thpinfo.com/2010/mygpoclient/
+"""
+    sys.exit(1)
+del mygpoclient
 
 # The User-Agent string for downloads
 user_agent = 'gPodder/%s (+%s)' % (__version__, __url__)
@@ -59,9 +68,11 @@ class UI(object):
 ui = UI()
 
 # D-Bus specific interface names
-dbus_bus_name = 'org.godder'
+dbus_bus_name = 'org.gpodder'
 dbus_gui_object_path = '/gui'
+dbus_podcasts_object_path = '/podcasts'
 dbus_interface = 'org.gpodder.interface'
+dbus_podcasts = 'org.gpodder.podcasts'
 
 # Set "win32" to True if we are on Windows
 win32 = (platform.system() == 'Windows')
@@ -72,6 +83,12 @@ textdomain = 'gpodder'
 locale_dir = gettext.bindtextdomain(textdomain)
 t = gettext.translation(textdomain, locale_dir, fallback=True)
 gettext = t.ugettext
+ngettext = t.ungettext
+if win32:
+    # Workaround for bug 650
+    from gtk.glade import bindtextdomain
+    bindtextdomain(textdomain, locale_dir)
+    del bindtextdomain
 del t
 
 # Set up textdomain for gtk.Builder (this accesses the C library functions)
@@ -104,7 +121,12 @@ icon_file = None
 STATE_NORMAL, STATE_DOWNLOADED, STATE_DELETED = range(3)
 
 # Default locations for configuration and data files
-home = os.path.expanduser(os.path.join('~', '.config', 'gpodder'))
+default_home = os.path.expanduser(os.path.join('~', '.config', 'gpodder'))
+home = os.environ.get('GPODDER_HOME', None)
+if home is None:
+    home = default_home
+else:
+    print >>sys.stderr, 'Using', home, 'to store data (GPODDER_HOME is set)'
 subscription_file = os.path.join(home, 'channels.opml')
 config_file = os.path.join(home, 'gpodder.conf')
 database_file = os.path.join(home, 'database.sqlite')
