@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -94,8 +94,12 @@ gPodderSettings = {
     'auto_update_frequency': (int, 20,
       ("The frequency (in minutes) at which gPodder will update all feeds "
         "if 'auto_update_feeds' is enabled.")),
+    'auto_cleanup_downloads': (bool, True,
+      ('Automatically removed cancelled and finished downloads from the list')),
     'episode_list_descriptions': (bool, True,
       ("Display the episode's description under the episode title in the GUI.")),
+    'episode_list_thumbnails': (bool, True,
+      ("Display thumbnails of downloaded image-feed episodes in the list")),
     'show_toolbar': (bool, True,
       ("Show the toolbar in the GUI's main window.")),
     'ipod_purge_old_episodes': (bool, False,
@@ -125,6 +129,8 @@ gPodderSettings = {
     'enable_notifications': (bool, True,
       ("Let gPodder use notification bubbles when it can completed certain "
         "tasks like downloading an episode or finishing syncing to a device.")),
+    'notifications_attach_to_tray': (str, 'minimized',
+      ("Attach notifications to tray icon. (always, minimized)")),
     'on_quit_ask': (bool, True,
       ("Ask the user to confirm quitting the application.")),
     'auto_download': (str, 'never',
@@ -212,6 +218,8 @@ gPodderSettings = {
       ('Internally used (current view mode)')),
     'podcast_list_hide_boring': (bool, False,
       ('Hide podcasts in the main window for which the episode list is empty')),
+    'podcast_list_view_all': (bool, False,
+      ('Show an additional entry in the podcast list that contains all episodes')),
 
     'audio_played_dbus': (bool, False,
       ('Set to True if the audio player notifies gPodder about played episodes')),
@@ -221,16 +229,26 @@ gPodderSettings = {
     'rotation_mode': (int, 0,
       ('Internally used on Maemo 5 for the current rotation mode')),
 
-    # Settings for my.gpodder.org
-    'my_gpodder_username': (str, '',
+    'youtube_preferred_fmt_id': (int, 18,
+      ('The preferred video format that should be downloaded from YouTube.')),
+
+    # my.gpodder.org general settings
+    'mygpo_username': (str, '',
       ("The user's gPodder web services username.")),
-    'my_gpodder_password': (str, '',
+    'mygpo_password': (str, '',
       ("The user's gPodder web services password.")),
-    'my_gpodder_autoupload': (bool, False,
-      ("Upload the user's podcast list to the gPodder web services when "
-        "gPodder is closed.")),
-    'my_gpodder_service': (str, 'http://my.gpodder.org',
-      ('The base URL of the my.gpodder.org service.')),
+    'mygpo_enabled': (bool, False,
+      ("Synchronize subscriptions with the web service.")),
+    'mygpo_server': (str, 'my.gpodder.org',
+      ('The hostname of the mygpo server in use.')),
+
+    # my.gpodder.org device-specific settings
+    'mygpo_device_uid': (str, util.get_hostname(),
+      ("The UID that is assigned to this installation.")),
+    'mygpo_device_caption': (str, _('gPodder on %s') % util.get_hostname(),
+      ("The human-readable name of this installation.")),
+    'mygpo_device_type': (str, 'desktop',
+      ("The type of the device gPodder is running on.")),
 
     # Paned position
     'paned_position': ( int, 200,
@@ -269,6 +287,11 @@ class Config(dict):
         self.load()
         self.apply_fixes()
 
+        download_dir = os.environ.get('GPODDER_DOWNLOAD_DIR', None)
+        if download_dir is not None:
+            log('Setting download_dir from environment: %s', download_dir, sender=self)
+            self.download_dir = download_dir
+
         atexit.register( self.__atexit)
 
     def apply_fixes(self):
