@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 import gpodder
 _ = gpodder.gettext
 
+import sys
+
 have_sqlite = True
 
 try:
@@ -36,9 +38,8 @@ except ImportError:
 
 # TODO: show a message box
 if not have_sqlite:
-    print "Please install pysqlite2 or upgrade to Python 2.5."
-    import sys
-    sys.exit()
+    print >>sys.stderr, 'Please install pysqlite2 or Python 2.5.'
+    sys.exit(1)
 
 from gpodder.liblogger import log
 from email.Utils import mktime_tz
@@ -48,21 +49,15 @@ from threading import RLock
 import string
 import re
 
-class Storage(object):
-    (STATE_NORMAL, STATE_DOWNLOADED, STATE_DELETED) = range(3)
+class Database(object):
     UNICODE_TRANSLATE = {ord(u'ö'): u'o', ord(u'ä'): u'a', ord(u'ü'): u'u'}
 
-    lock = None
-
-    def __init__(self):
-        self.settings = {}
+    def __init__(self, filename):
+        self.database_file = filename
         self.channel_map = {}
         self._db = None
         self.lock = RLock()
 
-    def setup(self, settings):
-        self.settings = settings
-
     def close(self):
         self.commit()
 
@@ -76,7 +71,7 @@ class Storage(object):
         self._db = None
 
     def log(self, message, *args, **kwargs):
-        if self.settings['gl'].config.log_sqlite:
+        if False:
             try:
                 message = message % args
                 log('%s', message, sender=self)
@@ -98,7 +93,7 @@ class Storage(object):
             AND id NOT IN
             (SELECT id FROM episodes WHERE channel_id = ?
             ORDER BY pubDate DESC LIMIT ?)"""
-        cur.execute(sql, (channel_id, self.STATE_DOWNLOADED, channel_id, max_episodes))
+        cur.execute(sql, (channel_id, gpodder.STATE_DOWNLOADED, channel_id, max_episodes))
 
         cur.close()
         self.lock.release()
@@ -127,7 +122,7 @@ class Storage(object):
     @property
     def db(self):
         if self._db is None:
-            self._db = sqlite.connect(self.settings['database'], check_same_thread=False)
+            self._db = sqlite.connect(self.database_file, check_same_thread=False)
             self._db.text_factory = str
             self._db.create_collation("UNICODE", self.db_sort_cmp)
             self.log('Connected')
@@ -174,7 +169,10 @@ class Storage(object):
             ("deleted", "INTEGER"),
             ("channel_is_locked", "INTEGER"),
             ("foldername", "TEXT"),
-            ("auto_foldername", "INTEGER")
+            ("auto_foldername", "INTEGER"),
+            ("release_expected", "INTEGER"),
+            ("release_deviation", "INTEGER"),
+            ("updated_timestamp", "INTEGER"),
             ))
 
         self.upgrade_table("episodes", (
@@ -211,42 +209,40 @@ class Storage(object):
 
         cur.execute("""CREATE TEMPORARY VIEW episodes_downloaded AS SELECT channel_id, COUNT(*) AS count FROM episodes WHERE state = 1 GROUP BY channel_id""")
         cur.execute("""CREATE TEMPORARY VIEW episodes_new AS SELECT channel_id, COUNT(*) AS count FROM episodes WHERE state = 0 AND played = 0 GROUP BY channel_id""")
-        cur.execute("""CREATE TEMPORARY VIEW episodes_unplayed AS SELECT channel_id, COUNT(*) AS count FROM episodes WHERE played = 0 AND state = %d GROUP BY channel_id""" % self.STATE_DOWNLOADED)
+        cur.execute("""CREATE TEMPORARY VIEW episodes_unplayed AS SELECT channel_id, COUNT(*) AS count FROM episodes WHERE played = 0 AND state = %d GROUP BY channel_id""" % gpodder.STATE_DOWNLOADED)
 
         # Make sure deleted episodes are played, to simplify querying statistics.
         try:
-            cur.execute("UPDATE episodes SET played = 1 WHERE state = ?", (self.STATE_DELETED, ))
+            cur.execute("UPDATE episodes SET played = 1 WHERE state = ?", (gpodder.STATE_DELETED, ))
         except OperationalError:
             pass
 
         cur.close()
         self.lock.release()
 
-    def get_channel_stat(self, url_or_id, state=None, is_played=None, is_locked=None):
-        where, params = ((),())
+    def get_channel_count(self, id):
+        """Given a channel ID, returns the statistics for it
 
-        if state is not None:
-            where += ("state = ?", )
-            params += (state, )
-        if is_played is not None:
-            where += ("played = ?", )
-            params += (is_played, )
-        if is_locked is not None:
-            where += ("locked = ?", )
-            params += (is_locked, )
-        if isinstance(url_or_id, int):
-            where += ("channel_id = ?", )
-            params += (url_or_id, )
-        else:
-            where += ("channel_id IN (SELECT id FROM channels WHERE url = ?)", )
-            params += (url_or_id, )
+        Returns a tuple (total, deleted, new, downloaded, unplayed)
+        """
+        total = self.__get__('SELECT COUNT(*) FROM episodes WHERE channel_id = ?', (id,))
+        deleted = self.__get__('SELECT COUNT(*) FROM episodes WHERE state = ? AND channel_id = ?', (gpodder.STATE_DELETED, id))
+        new = self.__get__('SELECT COUNT(*) FROM episodes WHERE state = ? AND played = ? AND channel_id = ?', (gpodder.STATE_NORMAL, False, id))
+        downloaded = self.__get__('SELECT COUNT(*) FROM episodes WHERE state = ? AND channel_id = ?', (gpodder.STATE_DOWNLOADED, id))
+        unplayed = self.__get__('SELECT COUNT(*) FROM episodes WHERE state = ? AND played = ? AND channel_id = ?', (gpodder.STATE_DOWNLOADED, False, id))
+        return (total, deleted, new, downloaded, unplayed)
 
-        self.log("get_channel_stats(%s)", url_or_id)
+    def get_total_count(self):
+        """Get statistics for all non-deleted channels
 
-        if len(where):
-            return self.__get__("SELECT COUNT(*) FROM episodes WHERE %s" % (' AND '.join(where)), params)
-        else:
-            return 0
+        Returns a tuple (total, deleted, new, downloaded, unplayed)
+        """
+        total = self.__get__('SELECT COUNT(*) FROM episodes WHERE channel_id IN (SELECT id FROM channels WHERE (deleted IS NULL OR deleted=0))')
+        deleted = self.__get__('SELECT COUNT(*) FROM episodes WHERE state = ? AND channel_id IN (SELECT id FROM channels WHERE (deleted IS NULL OR deleted=0))', (gpodder.STATE_DELETED,))
+        new = self.__get__('SELECT COUNT(*) FROM episodes WHERE state = ? AND played = ? AND channel_id IN (SELECT id FROM channels WHERE (deleted IS NULL OR deleted=0))', (gpodder.STATE_NORMAL, False,))
+        downloaded = self.__get__('SELECT COUNT(*) FROM episodes WHERE state = ? AND channel_id IN (SELECT id FROM channels WHERE (deleted IS NULL OR deleted=0))', (gpodder.STATE_DOWNLOADED,))
+        unplayed = self.__get__('SELECT COUNT(*) FROM episodes WHERE state = ? AND played = ? AND channel_id IN (SELECT id FROM channels WHERE (deleted IS NULL OR deleted=0))', (gpodder.STATE_DOWNLOADED, False,))
+        return (total, deleted, new, downloaded, unplayed)
 
     def load_channels(self, factory=None, url=None):
         """
@@ -276,7 +272,10 @@ class Storage(object):
                 etag,
                 channel_is_locked,
                 foldername,
-                auto_foldername
+                auto_foldername,
+                release_expected,
+                release_deviation,
+                updated_timestamp
             FROM
                 channels
             WHERE
@@ -307,6 +306,9 @@ class Storage(object):
                 'channel_is_locked': row[14],
                 'foldername': row[15],
                 'auto_foldername': row[16],
+                'release_expected': row[17],
+                'release_deviation': row[18],
+                'updated_timestamp': row[19],
                 }
 
             if row[0] in stats:
@@ -323,7 +325,7 @@ class Storage(object):
                 if factory is None:
                     result.append(channel)
                 else:
-                    result.append(factory(channel))
+                    result.append(factory(channel, self))
 
         cur.close()
         self.lock.release()
@@ -352,7 +354,7 @@ class Storage(object):
 
         return data
 
-    def save_channel(self, c, bulk=False):
+    def save_channel(self, c):
         if c.id is None:
             c.id = self.find_channel_id(c.url)
 
@@ -360,10 +362,10 @@ class Storage(object):
         self.log("save_channel((%s)%s)", c.id or "new", c.url)
 
         if c.id is None:
-            cur.execute("INSERT INTO channels (url, title, override_title, link, description, image, pubDate, sync_to_devices, device_playlist_name, username, password, last_modified, etag, channel_is_locked, foldername, auto_foldername) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", (c.url, c.title, c.override_title, c.link, c.description, c.image, self.__mktime__(c.pubDate), c.sync_to_devices, c.device_playlist_name, c.username, c.password, c.last_modified, c.etag, c.channel_is_locked, c.foldername, c.auto_foldername, ))
+            cur.execute("INSERT INTO channels (url, title, override_title, link, description, image, pubDate, sync_to_devices, device_playlist_name, username, password, last_modified, etag, channel_is_locked, foldername, auto_foldername, release_expected, release_deviation, updated_timestamp) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", (c.url, c.title, c.override_title, c.link, c.description, c.image, self.__mktime__(c.pubDate), c.sync_to_devices, c.device_playlist_name, c.username, c.password, c.last_modified, c.etag, c.channel_is_locked, c.foldername, c.auto_foldername, c.release_expected, c.release_deviation, c.updated_timestamp))
             self.channel_map[c.url] = cur.lastrowid
         else:
-            cur.execute("UPDATE channels SET url = ?, title = ?, override_title = ?, link = ?, description = ?, image = ?, pubDate = ?, sync_to_devices = ?, device_playlist_name = ?, username = ?, password = ?, last_modified = ?, etag = ?, channel_is_locked = ?, foldername = ?, auto_foldername = ?, deleted = 0 WHERE id = ?", (c.url, c.title, c.override_title, c.link, c.description, c.image, self.__mktime__(c.pubDate), c.sync_to_devices, c.device_playlist_name, c.username, c.password, c.last_modified, c.etag, c.channel_is_locked, c.foldername, c.auto_foldername, c.id, ))
+            cur.execute("UPDATE channels SET url = ?, title = ?, override_title = ?, link = ?, description = ?, image = ?, pubDate = ?, sync_to_devices = ?, device_playlist_name = ?, username = ?, password = ?, last_modified = ?, etag = ?, channel_is_locked = ?, foldername = ?, auto_foldername = ?, release_expected = ?, release_deviation = ?, updated_timestamp = ?, deleted = 0 WHERE id = ?", (c.url, c.title, c.override_title, c.link, c.description, c.image, self.__mktime__(c.pubDate), c.sync_to_devices, c.device_playlist_name, c.username, c.password, c.last_modified, c.etag, c.channel_is_locked, c.foldername, c.auto_foldername, c.release_expected, c.release_deviation, c.updated_timestamp, c.id, ))
 
         cur.close()
         self.lock.release()
@@ -382,9 +384,11 @@ class Storage(object):
                 del self.channel_map[channel.url]
         else:
             cur.execute("UPDATE channels SET deleted = 1 WHERE id = ?", (channel.id, ))
-            cur.execute("DELETE FROM episodes WHERE channel_id = ? AND state <> ?", (channel.id, self.STATE_DOWNLOADED))
+            cur.execute("DELETE FROM episodes WHERE channel_id = ? AND state <> ?", (channel.id, gpodder.STATE_DOWNLOADED))
 
         cur.close()
+        # Commit changes
+        self.db.commit()
         self.lock.release()
 
     def __read_episodes(self, factory=None, where=None, params=None, commit=True):
@@ -418,11 +422,11 @@ class Storage(object):
                 'id': row[13],
                 }
             if episode['state'] is None:
-                episode['state'] = self.STATE_NORMAL
+                episode['state'] = gpodder.STATE_NORMAL
             if factory is None:
                 result.append(episode)
             else:
-                result.append(factory(episode))
+                result.append(factory(episode, self))
 
         cur.close()
         self.lock.release()
@@ -440,7 +444,7 @@ class Storage(object):
                 (SELECT id FROM episodes WHERE channel_id = ?
                 ORDER BY pubDate DESC LIMIT ?)
                 ORDER BY pubDate DESC
-                """, params = (channel.id, self.STATE_DOWNLOADED, channel.id, limit, ))
+                """, params = (channel.id, gpodder.STATE_DOWNLOADED, channel.id, limit, ))
         else:
             return self.__read_episodes(factory = factory, where = " WHERE channel_id = ? AND state = ? ORDER BY pubDate DESC LIMIT ?", params = (channel.id, state, limit, ))
 
@@ -452,7 +456,7 @@ class Storage(object):
         else:
             return None
 
-    def save_episode(self, e, bulk=False):
+    def save_episode(self, e):
         if not e.guid:
             log('Refusing to save an episode without guid: %s', e)
             return
@@ -572,6 +576,14 @@ class Storage(object):
         """
         return self.__get__("SELECT id FROM channels WHERE foldername = ?", (foldername,)) is not None
 
+    def remove_foldername_if_deleted_channel(self, foldername):
+        cur = self.cursor(lock=True)
+        self.log('Setting foldername=NULL for folder "%s"', foldername)
+        cur.execute('UPDATE channels SET foldername=NULL ' + \
+                    'WHERE foldername=? AND deleted=1', (foldername,))
+        cur.close()
+        self.lock.release()
+
     def episode_filename_exists(self, filename):
         """
         Returns True if a filename for an episode exists.
@@ -593,8 +605,8 @@ class Storage(object):
 
     def force_last_new(self, channel):
         old = self.__get__("""SELECT COUNT(*) FROM episodes WHERE channel_id = ?
-            AND state IN (?, ?)""", (channel.id, self.STATE_DOWNLOADED,
-            self.STATE_DELETED))
+            AND state IN (?, ?)""", (channel.id, gpodder.STATE_DOWNLOADED,
+            gpodder.STATE_DELETED))
 
         cur = self.cursor(lock=True)
 
@@ -605,8 +617,8 @@ class Storage(object):
                 UPDATE episodes SET played = 1 WHERE channel_id = ?
                 AND played = 0 AND pubDate < (SELECT MAX(pubDate)
                 FROM episodes WHERE channel_id = ? AND state IN (?, ?))""",
-                (channel.id, channel.id, self.STATE_DOWNLOADED,
-                self.STATE_DELETED, ))
+                (channel.id, channel.id, gpodder.STATE_DOWNLOADED,
+                gpodder.STATE_DELETED, ))
         else:
             cur.execute("""
                 UPDATE episodes SET played = 1 WHERE channel_id = ?
@@ -646,7 +658,17 @@ class Storage(object):
         """
         cur = self.cursor(lock=True)
         log('Deleting old episodes from channel #%d' % channel_id)
-        cur.execute("DELETE FROM episodes WHERE channel_id = ? AND state != ?", (channel_id, self.STATE_DOWNLOADED, ))
+        cur.execute("DELETE FROM episodes WHERE channel_id = ? AND state != ?", (channel_id, gpodder.STATE_DOWNLOADED, ))
+        self.lock.release()
+
+    def delete_episode_by_guid(self, guid, channel_id):
+        """
+        Deletes episodes that have a specific GUID for
+        a given channel. Used after feed updates for
+        episodes that have disappeared from the feed.
+        """
+        cur = self.cursor(lock=True)
+        cur.execute('DELETE FROM episodes WHERE channel_id = ? AND guid = ?', \
+                (channel_id, guid))
         self.lock.release()
 
-db = Storage()
