@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 
 
 #
-#  download.py -- Download client using DownloadStatusManager
+#  download.py -- Download queue management
 #  Thomas Perl <thp@perli.net>   2007-09-15
 #
 #  Based on libwget.py (2005-10-29)
@@ -28,10 +28,8 @@
 from __future__ import with_statement
 
 from gpodder.liblogger import log
-from gpodder.libgpodder import gl
-from gpodder.dbsqlite import db
 from gpodder import util
-from gpodder import resolver
+from gpodder import youtube
 import gpodder
 
 import threading
@@ -43,10 +41,45 @@ import os
 import time
 import collections
 
+import mimetypes
+import email
+import email.Header
+
 from xml.sax import saxutils
 
 _ = gpodder.gettext
 
+def get_header_param(headers, param, header_name):
+    """Extract a HTTP header parameter from a dict
+
+    Uses the "email" module to retrieve parameters
+    from HTTP headers. This can be used to get the
+    "filename" parameter of the "content-disposition"
+    header for downloads to pick a good filename.
+
+    Returns None if the filename cannot be retrieved.
+    """
+    try:
+        headers_string = ['%s:%s'%(k,v) for k,v in headers.items()]
+        msg = email.message_from_string('\n'.join(headers_string))
+        if header_name in msg:
+            value = msg.get_param(param, header=header_name)
+            if value is None:
+                return None
+            decoded_list = email.Header.decode_header(value)
+            value = []
+            for part, encoding in decoded_list:
+                if encoding:
+                    value.append(part.decode(encoding))
+                else:
+                    value.append(unicode(part))
+            return u''.join(value)
+    except Exception, e:
+        log('Error trying to get %s from %s: %s', \
+                param, header_name, str(e), traceback=True)
+
+    return None
+
 class ContentRange(object):
     # Based on:
     # http://svn.pythonpaste.org/Paste/WebOb/trunk/webob/byterange.py
@@ -150,6 +183,7 @@ class ContentRange(object):
 
 
 class DownloadCancelledException(Exception): pass
+class AuthenticationError(Exception): pass
 
 class gPodderDownloadHTTPError(Exception):
     def __init__(self, url, error_code, error_message):
@@ -160,18 +194,15 @@ class gPodderDownloadHTTPError(Exception):
 class DownloadURLOpener(urllib.FancyURLopener):
     version = gpodder.user_agent
 
-    def __init__( self, channel):
-        if gl.config.proxy_use_environment:
-            proxies = None
-        else:
-            proxies = {}
-            if gl.config.http_proxy:
-                proxies['http'] = gl.config.http_proxy
-            if gl.config.ftp_proxy:
-                proxies['ftp'] = gl.config.ftp_proxy
+    # Sometimes URLs are not escaped correctly - try to fix them
+    # (see RFC2396; Section 2.4.3. Excluded US-ASCII Characters)
+    # FYI: The omission of "%" in the list is to avoid double escaping!
+    ESCAPE_CHARS = dict((ord(c), u'%%%x'%ord(c)) for c in u' <>#"{}|\\^[]`')
 
+    def __init__( self, channel):
         self.channel = channel
-        urllib.FancyURLopener.__init__( self, proxies)
+        self._auth_retry_counter = 0
+        urllib.FancyURLopener.__init__(self, None)
 
     def http_error_default(self, url, fp, errcode, errmsg, headers):
         """
@@ -214,12 +245,11 @@ class DownloadURLOpener(urllib.FancyURLopener):
         return urllib.addinfourl(fp, headers, 'http:' + url)
 
     def retrieve_resume(self, url, filename, reporthook=None, data=None):
-        """retrieve_resume(url) returns (filename, headers) for a local object
-        or (tempfilename, headers) for a remote object.
+        """Download files from an URL; return (headers, real_url)
 
-        The filename argument is REQUIRED (no tempfile creation code here!)
-
-        Additionally resumes a download if the local filename exists"""
+        Resumes a download if the local filename exists and
+        the server supports download resuming.
+        """
 
         current_size = 0
         tfp = None
@@ -228,7 +258,8 @@ class DownloadURLOpener(urllib.FancyURLopener):
                 current_size = os.path.getsize(filename)
                 tfp = open(filename, 'ab')
                 #If the file exists, then only download the remainder
-                self.addheader('Range', 'bytes=%s-' % (current_size))
+                if current_size > 0:
+                    self.addheader('Range', 'bytes=%s-' % (current_size))
             except:
                 log('Cannot open file for resuming: %s', filename, sender=self, traceback=True)
                 tfp = None
@@ -237,6 +268,11 @@ class DownloadURLOpener(urllib.FancyURLopener):
         if tfp is None:
             tfp = open(filename, 'wb')
 
+        # Fix a problem with bad URLs that are not encoded correctly (bug 549)
+        url = url.decode('ascii', 'ignore')
+        url = url.translate(self.ESCAPE_CHARS)
+        url = url.encode('ascii')
+
         url = urllib.unwrap(urllib.toBytes(url))
         fp = self.open(url, data)
         headers = fp.info()
@@ -254,11 +290,7 @@ class DownloadURLOpener(urllib.FancyURLopener):
                 current_size = 0
                 log('Cannot resume. Missing or wrong Content-Range header (RFC2616)', sender=self)
 
-
-        # gPodder TODO: we can get the real url via fp.geturl() here
-        #               (if anybody wants to fix filenames in the future)
-
-        result = filename, headers
+        result = headers, fp.geturl()
         bs = 1024*8
         size = -1
         read = current_size
@@ -267,8 +299,11 @@ class DownloadURLOpener(urllib.FancyURLopener):
             if "content-length" in headers:
                 size = int(headers["Content-Length"]) + current_size
             reporthook(blocknum, bs, size)
-        while 1:
-            block = fp.read(bs)
+        while read < size or size == -1:
+            if size == -1:
+                block = fp.read(bs)
+            else:
+                block = fp.read(min(size-read, bs))
             if block == "":
                 break
             read += len(block)
@@ -291,32 +326,33 @@ class DownloadURLOpener(urllib.FancyURLopener):
 # end code based on urllib.py
 
     def prompt_user_passwd( self, host, realm):
+        # Keep track of authentication attempts, fail after the third one
+        self._auth_retry_counter += 1
+        if self._auth_retry_counter > 3:
+            raise AuthenticationError(_('Wrong username/password'))
+
         if self.channel.username or self.channel.password:
             log( 'Authenticating as "%s" to "%s" for realm "%s".', self.channel.username, host, realm, sender = self)
             return ( self.channel.username, self.channel.password )
 
-        return ( None, None )
+        return (None, None)
 
 
 class DownloadQueueWorker(threading.Thread):
-    def __init__(self, queue, exit_callback):
+    def __init__(self, queue, exit_callback, continue_check_callback):
         threading.Thread.__init__(self)
         self.queue = queue
         self.exit_callback = exit_callback
-        self.cancelled = False
-
-    def stop_accepting_tasks(self):
-        """
-        When this is called, the worker will not accept new tasks,
-        but quit when the current task has been finished.
-        """
-        if not self.cancelled:
-            self.cancelled = True
-            log('%s stopped accepting tasks.', self.getName(), sender=self)
+        self.continue_check_callback = continue_check_callback
 
     def run(self):
         log('Running new thread: %s', self.getName(), sender=self)
-        while not self.cancelled:
+        while True:
+            # Check if this thread is allowed to continue accepting tasks
+            if not self.continue_check_callback(self):
+                log('%s must not accept new tasks.', self.getName(), sender=self)
+                return
+
             try:
                 task = self.queue.pop()
                 log('%s is processing: %s', self.getName(), task, sender=self)
@@ -328,8 +364,8 @@ class DownloadQueueWorker(threading.Thread):
 
 
 class DownloadQueueManager(object):
-    def __init__(self, download_status_manager):
-        self.download_status_manager = download_status_manager
+    def __init__(self, config):
+        self._config = config
         self.tasks = collections.deque()
 
         self.worker_threads_access = threading.RLock()
@@ -339,38 +375,41 @@ class DownloadQueueManager(object):
         with self.worker_threads_access:
             self.worker_threads.remove(worker_thread)
 
-    def spawn_and_retire_threads(self, request_new_thread=False):
+    def __continue_check_callback(self, worker_thread):
         with self.worker_threads_access:
-            if len(self.worker_threads) > gl.config.max_downloads and \
-                    gl.config.max_downloads_enabled:
-                # Tell the excessive amount of oldest worker threads to quit, but keep at least one
-                count = min(len(self.worker_threads)-1, len(self.worker_threads)-gl.config.max_downloads)
-                for worker in self.worker_threads[:count]:
-                    worker.stop_accepting_tasks()
+            if len(self.worker_threads) > self._config.max_downloads and \
+                    self._config.max_downloads_enabled:
+                self.worker_threads.remove(worker_thread)
+                return False
+            else:
+                return True
 
-            if request_new_thread and (len(self.worker_threads) == 0 or \
-                    len(self.worker_threads) < gl.config.max_downloads or \
-                    not gl.config.max_downloads_enabled):
+    def spawn_threads(self):
+        with self.worker_threads_access:
+            if not len(self.tasks):
+                return
+
+            if len(self.worker_threads) == 0 or \
+                    len(self.worker_threads) < self._config.max_downloads or \
+                    not self._config.max_downloads_enabled:
                 # We have to create a new thread here, there's work to do
                 log('I am going to spawn a new worker thread.', sender=self)
-                worker = DownloadQueueWorker(self.tasks, self.__exit_callback)
+                worker = DownloadQueueWorker(self.tasks, self.__exit_callback, \
+                        self.__continue_check_callback)
                 self.worker_threads.append(worker)
                 worker.start()
 
-    def add_resumed_task(self, task):
-        """Simply add the task without starting the download"""
-        self.download_status_manager.register_task(task)
+    def are_queued_or_active_tasks(self):
+        with self.worker_threads_access:
+            return len(self.worker_threads) > 0
 
     def add_task(self, task):
-        if task.status == DownloadTask.INIT:
-            # This task is fresh, so add it to our status manager
-            self.download_status_manager.register_task(task)
-        else:
+        if task.status != DownloadTask.INIT:
             # This task is old so update episode from db
             task.episode.reload_from_db()
         task.status = DownloadTask.QUEUED
         self.tasks.appendleft(task)
-        self.spawn_and_retire_threads(request_new_thread=True)
+        self.spawn_threads()
 
 
 class DownloadTask(object):
@@ -378,7 +417,7 @@ class DownloadTask(object):
 
     You can create a new download task like this:
 
-        task = DownloadTask(episode)
+        task = DownloadTask(episode, gpodder.config.Config(CONFIGFILE))
         task.status = DownloadTask.QUEUED
         task.run()
 
@@ -483,15 +522,19 @@ class DownloadTask(object):
         if self.status != self.DONE:
             util.delete_file(self.tempname)
 
-    def __init__(self, episode):
+    def __init__(self, episode, config):
         self.__status = DownloadTask.INIT
         self.__status_changed = True
         self.__episode = episode
+        self._config = config
+
+        # Set names for the downloads list
+        self.markup_name = saxutils.escape(self.__episode.title)
+        self.markup_podcast_name = saxutils.escape(self.__episode.channel.title)
 
         # Create the target filename and save it in the database
         self.filename = self.__episode.local_filename(create=True)
         self.tempname = self.filename + '.partial'
-        db.commit()
 
         self.total_size = self.__episode.length
         self.speed = 0.0
@@ -501,8 +544,8 @@ class DownloadTask(object):
         # Variables for speed limit and speed calculation
         self.__start_time = 0
         self.__start_blocks = 0
-        self.__limit_rate_value = gl.config.limit_rate_value
-        self.__limit_rate = gl.config.limit_rate
+        self.__limit_rate_value = self._config.limit_rate_value
+        self.__limit_rate = self._config.limit_rate
 
         # If the tempname already exists, set progress accordingly
         if os.path.exists(self.tempname):
@@ -539,18 +582,18 @@ class DownloadTask(object):
             now = time.time()
             if self.__start_time > 0:
                 # Has rate limiting been enabled or disabled?                
-                if self.__limit_rate != gl.config.limit_rate: 
+                if self.__limit_rate != self._config.limit_rate: 
                     # If it has been enabled then reset base time and block count                    
-                    if gl.config.limit_rate:
+                    if self._config.limit_rate:
                         self.__start_time = now
                         self.__start_blocks = count
-                    self.__limit_rate = gl.config.limit_rate
+                    self.__limit_rate = self._config.limit_rate
                     
                 # Has the rate been changed and are we currently limiting?            
-                if self.__limit_rate_value != gl.config.limit_rate_value and self.__limit_rate: 
+                if self.__limit_rate_value != self._config.limit_rate_value and self.__limit_rate: 
                     self.__start_time = now
                     self.__start_blocks = count
-                    self.__limit_rate_value = gl.config.limit_rate_value
+                    self.__limit_rate_value = self._config.limit_rate_value
 
                 passed = now - self.__start_time
                 if passed > 0:
@@ -565,10 +608,10 @@ class DownloadTask(object):
 
             self.speed = float(speed)
 
-            if gl.config.limit_rate and speed > gl.config.limit_rate_value:
+            if self._config.limit_rate and speed > self._config.limit_rate_value:
                 # calculate the time that should have passed to reach
                 # the desired download rate and wait if necessary
-                should_have_passed = float((count-self.__start_blocks)*blockSize)/(gl.config.limit_rate_value*1024.0)
+                should_have_passed = float((count-self.__start_blocks)*blockSize)/(self._config.limit_rate_value*1024.0)
                 if should_have_passed > passed:
                     # sleep a maximum of 10 seconds to not cause time-outs
                     delay = min(10.0, float(should_have_passed-passed))
@@ -582,6 +625,8 @@ class DownloadTask(object):
         # If the download has already been cancelled, skip it
         if self.status == DownloadTask.CANCELLED:
             util.delete_file(self.tempname)
+            self.progress = 0.0
+            self.speed = 0.0
             return False
 
         # We only start this download if its status is "queued"
@@ -593,9 +638,10 @@ class DownloadTask(object):
 
         try:
             # Resolve URL and start downloading the episode
-            url = resolver.get_real_download_url(self.__episode.url)
+            url = youtube.get_real_download_url(self.__episode.url, \
+                    self._config.youtube_preferred_fmt_id)
             downloader =  DownloadURLOpener(self.__episode.channel)
-            (unused, headers) = downloader.retrieve_resume(url,
+            headers, real_url = downloader.retrieve_resume(url, \
                     self.tempname, reporthook=self.status_updated)
 
             new_mimetype = headers.get('content-type', self.__episode.mimetype)
@@ -611,6 +657,24 @@ class DownloadTask(object):
                 if old_extension != new_extension:
                     self.filename = self.__episode.local_filename(create=True, force_update=True)
 
+            # TODO: Check if "real_url" is different from "url" and if it is,
+            #       see if we can get a better episode filename out of it
+
+            # Look at the Content-disposition header; use if if available
+            disposition_filename = get_header_param(headers, \
+                    'filename', 'content-disposition')
+
+            if disposition_filename is not None:
+                # The server specifies a download filename - try to use it
+                disposition_filename = os.path.basename(disposition_filename)
+                self.filename = self.__episode.local_filename(create=True, \
+                        force_update=True, template=disposition_filename)
+                new_mimetype, encoding = mimetypes.guess_type(self.filename)
+                if new_mimetype is not None:
+                    log('Using content-disposition mimetype: %s',
+                            new_mimetype, sender=self)
+                    self.__episode.set_mimetype(new_mimetype, commit=True)
+
             shutil.move(self.tempname, self.filename)
 
             # Get the _real_ filesize once we actually have the file
@@ -618,28 +682,34 @@ class DownloadTask(object):
             self.__episode.channel.addDownloadedItem(self.__episode)
             
             # If a user command has been defined, execute the command setting some environment variables
-            if len(gl.config.cmd_download_complete) > 0:
+            if len(self._config.cmd_download_complete) > 0:
                 os.environ["GPODDER_EPISODE_URL"]=self.__episode.url or ''
                 os.environ["GPODDER_EPISODE_TITLE"]=self.__episode.title or ''
                 os.environ["GPODDER_EPISODE_FILENAME"]=self.filename or ''
                 os.environ["GPODDER_EPISODE_PUBDATE"]=str(int(self.__episode.pubDate))
                 os.environ["GPODDER_EPISODE_LINK"]=self.__episode.link or ''
                 os.environ["GPODDER_EPISODE_DESC"]=self.__episode.description or ''
-                util.run_external_command(gl.config.cmd_download_complete)
+                os.environ["GPODDER_CHANNEL_TITLE"]=self.__episode.channel.title or ''
+                util.run_external_command(self._config.cmd_download_complete)
         except DownloadCancelledException:
             log('Download has been cancelled/paused: %s', self, sender=self)
             if self.status == DownloadTask.CANCELLED:
                 util.delete_file(self.tempname)
                 self.progress = 0.0
                 self.speed = 0.0
-        except IOError, ioe:
-            log( 'Error "%s" while downloading "%s": %s', ioe.strerror, self.__episode.title, ioe.filename, sender=self)
+        except urllib.ContentTooShortError, ctse:
             self.status = DownloadTask.FAILED
-            self.error_message = _('I/O Error: %s: %s') % (ioe.strerror, ioe.filename)
+            self.error_message = _('Missing content from server')
+        except IOError, ioe:
+            log( 'Error "%s" while downloading "%s": %s', ioe.strerror, self.__episode.title, ioe.filename, sender=self, traceback=True)
+            self.status = DownloadTask.FAILED
+            d = {'error': ioe.strerror, 'filename': ioe.filename}
+            self.error_message = _('I/O Error: %(error)s: %(filename)s') % d
         except gPodderDownloadHTTPError, gdhe:
             log( 'HTTP error %s while downloading "%s": %s', gdhe.error_code, self.__episode.title, gdhe.error_message, sender=self)
             self.status = DownloadTask.FAILED
-            self.error_message = _('HTTP Error %s: %s') % (gdhe.error_code, gdhe.error_message)
+            d = {'code': gdhe.error_code, 'message': gdhe.error_message}
+            self.error_message = _('HTTP Error %(code)s: %(message)s') % d
         except Exception, e:
             self.status = DownloadTask.FAILED
             self.error_message = _('Error: %s') % (e.message,)
@@ -647,6 +717,9 @@ class DownloadTask(object):
         if self.status == DownloadTask.DOWNLOADING:
             # Everything went well - we're done
             self.status = DownloadTask.DONE
+            if self.total_size <= 0:
+                self.total_size = util.calculate_size(self.filename)
+                log('Total size updated to %d', self.total_size, sender=self)
             self.progress = 1.0
             return True
         
