@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@ from xml.sax import saxutils
 
 import gpodder
 _ = gpodder.gettext
+N_ = gpodder.ngettext
 
 from gpodder import util
 from gpodder.liblogger import log
@@ -281,10 +282,8 @@ class gPodderEpisodeSelector(BuilderWidget):
             text = []
             if count == 0: 
                 text.append(_('Nothing selected'))
-            elif count == 1:
-                text.append(_('One episode'))
             else:
-                text.append(_('%d episodes') % count)
+                text.append(N_('%d episode', '%d episodes', count) % count)
             if total_size > 0: 
                 text.append(_('size: %s') % util.format_filesize(total_size))
             self.labelTotalSize.set_text(', '.join(text))
