@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@ class BuilderWidget(GtkBuilderWidget):
 
     def __init__(self, parent, **kwargs):
         self._window_iconified = False
+        self._window_visible = False
 
         # Enable support for portrait mode changes on Maemo 5
         if gpodder.ui.fremantle:
@@ -74,6 +75,11 @@ class BuilderWidget(GtkBuilderWidget):
             self.main_window.connect('window-state-event', \
                     self._on_window_state_event_iconified)
 
+        # Enable support for tracking visibility state
+        if gpodder.ui.desktop:
+            self.main_window.connect('visibility-notify-event', \
+                        self._on_window_state_event_visibility)
+
         # Set widgets to finger-friendly mode if on Maemo
         for widget_name in self.finger_friendly_widgets:
             if hasattr(self, widget_name):
@@ -140,6 +146,14 @@ class BuilderWidget(GtkBuilderWidget):
 
         return False
 
+    def _on_window_state_event_visibility(self, widget, event):
+        if event.state & gtk.gdk.VISIBILITY_FULLY_OBSCURED:
+            self._window_visible = False
+        else:
+            self._window_visible = True
+
+        return False
+
     def _on_window_state_event_iconified(self, widget, event):
         if event.new_window_state & gtk.gdk.WINDOW_STATE_ICONIFIED:
             if not self._window_iconified:
@@ -185,6 +199,9 @@ class BuilderWidget(GtkBuilderWidget):
                 hildon.hildon_banner_show_information(self.main_window, \
                         '', message)
         else:
+            # XXX: Dirty hack to get access to the gPodder-specific config object
+            config = getattr(self, '_config', getattr(self, 'config', None))
+
             if important:
                 dlg = gtk.MessageDialog(self.main_window, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_OK)
                 if title:
@@ -194,25 +211,40 @@ class BuilderWidget(GtkBuilderWidget):
                     dlg.set_markup('<span weight="bold" size="larger">%s</span>' % (message))
                 dlg.run()
                 dlg.destroy()
-            elif pynotify is not None:
-                if title is None:
-                    title = 'gPodder'
-                notification = pynotify.Notification(title, message, gpodder.icon_file)
+            elif config is not None and config.enable_notifications:
+                if pynotify is not None:
+                    if title is None:
+                        title = 'gPodder'
+                    notification = pynotify.Notification(title, message, gpodder.icon_file)
+                    _notify_at_tray = False
+                    _notify_when = config.notifications_attach_to_tray
+                    _notify_attach = notification.attach_to_status_icon
 
-                if widget and isinstance(widget, gtk.Widget):
+                    if _notify_when in ('minimized','always'):
+                        if getattr(self,'tray_icon',None) and self.tray_icon.is_embedded():
+                            _notify_at_tray = True
+
+                    if _notify_at_tray and _notify_when == 'always':
+                        _notify_attach(self.tray_icon)
+                    elif not self._window_iconified and self.main_window.is_active:
+                        if self._window_visible:
+                            if widget and isinstance(widget, gtk.Widget):
+                                if not widget.window:
+                                    widget = self.main_window
+                                notification.attach_to_widget(widget)
+                        elif _notify_at_tray and _notify_when == 'minimized':
+                            _notify_attach(self.tray_icon)
+                    elif _notify_at_tray and _notify_when == 'minimized':
+                        _notify_attach(self.tray_icon)
+
+                    notification.show()
+                elif widget and isinstance(widget, gtk.Widget):
                     if not widget.window:
                         widget = self.main_window
-                    notification.attach_to_widget(widget)
-
-                notification.show()
-            else:
-                if widget and isinstance(widget, gtk.Widget):
-                    if not widget.window:
+                    else:
                         widget = self.main_window
-                else:
-                    widget = self.main_window
-                notification = NotificationWindow(message, title, important=False, widget=widget)
-                notification.show_timeout()
+                    notification = NotificationWindow(message, title, important=False, widget=widget)
+                    notification.show_timeout()
 
     def set_finger_friendly(self, widget):
         """
