@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,19 +28,21 @@ import gpodder
 _ = gpodder.gettext
 
 from gpodder import util
+from gpodder.liblogger import log
 
 from gpodder.gtkui import draw
 
+import os
 import gtk
 import xml.sax.saxutils
 
 class EpisodeListModel(gtk.ListStore):
     C_URL, C_TITLE, C_FILESIZE_TEXT, C_EPISODE, C_STATUS_ICON, \
-            C_PUBLISHED_TEXT, C_DESCRIPTION, C_DESCRIPTION_STRIPPED, \
+            C_PUBLISHED_TEXT, C_DESCRIPTION, C_TOOLTIP, \
             C_VIEW_SHOW_UNDELETED, C_VIEW_SHOW_DOWNLOADED, \
             C_VIEW_SHOW_UNPLAYED = range(11)
 
-    SEARCH_COLUMNS = (C_TITLE, C_DESCRIPTION_STRIPPED)
+    SEARCH_COLUMNS = (C_TITLE, C_DESCRIPTION)
 
     VIEW_ALL, VIEW_UNDELETED, VIEW_DOWNLOADED, VIEW_UNPLAYED = range(4)
 
@@ -60,6 +62,7 @@ class EpisodeListModel(gtk.ListStore):
         self._icon_cache = {}
         self.ICON_AUDIO_FILE = ICON('audio-x-generic')
         self.ICON_VIDEO_FILE = ICON('video-x-generic')
+        self.ICON_IMAGE_FILE = ICON('image-x-generic')
         self.ICON_GENERIC_FILE = ICON('text-x-generic')
         self.ICON_DOWNLOADING = gtk.STOCK_GO_DOWN
         self.ICON_DELETED = gtk.STOCK_DELETE
@@ -123,9 +126,15 @@ class EpisodeListModel(gtk.ListStore):
     def get_search_term(self):
         return self._search_term
 
+    def _format_description(self, episode, include_description=False, is_downloading=None):
+        if include_description:
+            return '%s\n<small>%s</small>' % (xml.sax.saxutils.escape(episode.title),
+                    xml.sax.saxutils.escape(episode.one_line_description()))
+        else:
+            return xml.sax.saxutils.escape(episode.title)
 
     def add_from_channel(self, channel, downloading=None, \
-            include_description=False):
+            include_description=False, generate_thumbnails=False):
         """
         Add episode from the given channel to this model.
         Downloading should be a callback.
@@ -133,7 +142,6 @@ class EpisodeListModel(gtk.ListStore):
         is to be added to the episode row, or False if not)
         """
         def insert_and_update(episode):
-            description = episode.format_episode_row_markup(include_description)
             description_stripped = util.remove_html_tags(episode.description)
 
             iter = self.append()
@@ -143,31 +151,35 @@ class EpisodeListModel(gtk.ListStore):
                     self.C_FILESIZE_TEXT, self._format_filesize(episode), \
                     self.C_EPISODE, episode, \
                     self.C_PUBLISHED_TEXT, episode.cute_pubdate(), \
-                    self.C_DESCRIPTION, description, \
-                    self.C_DESCRIPTION_STRIPPED, description_stripped)
+                    self.C_TOOLTIP, description_stripped)
 
-            self.update_by_iter(iter, downloading, include_description)
+            self.update_by_iter(iter, downloading, include_description, generate_thumbnails)
 
         for episode in channel.get_all_episodes():
             util.idle_add(insert_and_update, episode)
 
-    def update_all(self, downloading=None, include_description=False):
+    def update_all(self, downloading=None, include_description=False, \
+            generate_thumbnails=False):
         for row in self:
-            self.update_by_iter(row.iter, downloading, include_description)
+            self.update_by_iter(row.iter, downloading, include_description, \
+                    generate_thumbnails)
 
-    def update_by_urls(self, urls, downloading=None, include_description=False):
+    def update_by_urls(self, urls, downloading=None, include_description=False, \
+            generate_thumbnails=False):
         for row in self:
             if row[self.C_URL] in urls:
-                self.update_by_iter(row.iter, downloading, include_description)
+                self.update_by_iter(row.iter, downloading, include_description, \
+                        generate_thumbnails)
 
     def update_by_filter_iter(self, iter, downloading=None, \
-            include_description=False):
+            include_description=False, generate_thumbnails=False):
         # Convenience function for use by "outside" methods that use iters
         # from the filtered episode list model (i.e. all UI things normally)
         self.update_by_iter(self._filter.convert_iter_to_child_iter(iter), \
-                downloading, include_description)
+                downloading, include_description, generate_thumbnails)
 
-    def update_by_iter(self, iter, downloading=None, include_description=False):
+    def update_by_iter(self, iter, downloading=None, include_description=False, \
+            generate_thumbnails=False):
         episode = self.get_value(iter, self.C_EPISODE)
         episode.reload_from_db()
 
@@ -180,24 +192,30 @@ class EpisodeListModel(gtk.ListStore):
         show_padlock = False
         show_missing = False
         status_icon = None
+        status_icon_to_build_from_file = False
+        tooltip = ''
         view_show_undeleted = True
         view_show_downloaded = False
         view_show_unplayed = False
 
         if downloading is not None and downloading(episode):
+            tooltip = _('Downloading')
             status_icon = self.ICON_DOWNLOADING
             view_show_downloaded = True
             view_show_unplayed = True
         else:
             if episode.state == gpodder.STATE_DELETED:
+                tooltip = _('Deleted')
                 status_icon = self.ICON_DELETED
                 view_show_undeleted = False
             elif episode.state == gpodder.STATE_NORMAL and \
                     not episode.is_played:
+                tooltip = _('New episode')
                 status_icon = self.ICON_NEW
                 view_show_downloaded = True
                 view_show_unplayed = True
             elif episode.state == gpodder.STATE_DOWNLOADED:
+                tooltip = []
                 view_show_downloaded = True
                 view_show_unplayed = not episode.is_played
                 show_bullet = not episode.is_played
@@ -206,32 +224,108 @@ class EpisodeListModel(gtk.ListStore):
 
                 file_type = episode.file_type()
                 if file_type == 'audio':
+                    tooltip.append(_('Downloaded episode'))
                     status_icon = self.ICON_AUDIO_FILE
                 elif file_type == 'video':
+                    tooltip.append(_('Downloaded video episode'))
                     status_icon = self.ICON_VIDEO_FILE
+                elif file_type == 'image':
+                    tooltip.append(_('Downloaded image'))
+                    status_icon = self.ICON_IMAGE_FILE
+
+                    # Optional thumbnailing for image downloads
+                    if generate_thumbnails:
+                        image_path = episode.local_filename(create=False, check_only=True)
+                        if image_path is not None:
+                            # set the status icon to the path itself (that
+                            # should be a good identifier anyway)
+                            status_icon = image_path
+                            status_icon_to_build_from_file = True
                 else:
+                    tooltip.append(_('Downloaded file'))
                     status_icon = self.ICON_GENERIC_FILE
 
+                if show_missing:
+                    tooltip.append(_('missing file'))
+                else:
+                    if show_bullet:
+                        if file_type == 'image':
+                            tooltip.append(_('never displayed'))
+                        else:
+                            tooltip.append(_('never played'))
+                    else:
+                        if file_type == 'image':
+                            tooltip.append(_('displayed'))
+                        else:
+                            tooltip.append(_('played'))
+                    if show_padlock:
+                        tooltip.append(_('deletion prevented'))
+
+                tooltip = ', '.join(tooltip)
+
         if status_icon is not None:
             status_icon = self._get_tree_icon(status_icon, show_bullet, \
-                    show_padlock, show_missing, icon_size)
+                    show_padlock, show_missing, icon_size, status_icon_to_build_from_file)
 
+        description = self._format_description(episode, include_description, downloading)
         self.set(iter, \
                 self.C_STATUS_ICON, status_icon, \
                 self.C_VIEW_SHOW_UNDELETED, view_show_undeleted, \
                 self.C_VIEW_SHOW_DOWNLOADED, view_show_downloaded, \
-                self.C_VIEW_SHOW_UNPLAYED, view_show_unplayed)
+                self.C_VIEW_SHOW_UNPLAYED, view_show_unplayed, \
+                self.C_DESCRIPTION, description, \
+                self.C_TOOLTIP, tooltip)
 
+    def _get_icon_from_image(self,image_path, icon_size):
+        """
+        Load an local image file and transform it into an icon.
+
+        Return a pixbuf scaled to the desired size and may return None
+        if the icon creation is impossible (file not found etc).
+        """
+        if not os.path.exists(image_path):
+            return None
+        # load image from disc (code adapted from CoverDownloader
+        # except that no download is needed here)
+        loader = gtk.gdk.PixbufLoader()
+        pixbuf = None
+        try:
+            loader.write(open(image_path, 'rb').read())
+            loader.close()
+            pixbuf = loader.get_pixbuf()
+        except:
+            log('Data error while loading image %s', image_path, sender=self)
+            return None
+        # Now scale the image with ratio (copied from _resize_pixbuf_keep_ratio)
+        # Resize if too wide
+        if pixbuf.get_width() > icon_size:
+            f = float(icon_size)/pixbuf.get_width()
+            (width, height) = (int(pixbuf.get_width()*f), int(pixbuf.get_height()*f))
+            pixbuf = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
+        # Resize if too high
+        if pixbuf.get_height() > icon_size:
+            f = float(icon_size)/pixbuf.get_height()
+            (width, height) = (int(pixbuf.get_width()*f), int(pixbuf.get_height()*f))
+            pixbuf = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
+        return pixbuf
+        
+        
     def _get_tree_icon(self, icon_name, add_bullet=False, \
-            add_padlock=False, add_missing=False, icon_size=32):
+            add_padlock=False, add_missing=False, icon_size=32, \
+            build_icon_from_file = False):
         """
         Loads an icon from the current icon theme at the specified
         size, suitable for display in a gtk.TreeView. Additional
         emblems can be added on top of the icon.
 
         Caching is used to speed up the icon lookup.
+        
+        The `build_icon_from_file` argument indicates (when True) that
+        the icon has to be created on the fly from a given image
+        file. The `icon_name` argument is then interpreted as the path
+        to this file. Those specific icons will *not be cached*.
         """
-
+        
         # Add all variables that modify the appearance of the icon, so
         # our cache does not return the same icons for different requests
         cache_id = (icon_name, add_bullet, add_padlock, add_missing, icon_size)
@@ -242,7 +336,10 @@ class EpisodeListModel(gtk.ListStore):
         icon_theme = gtk.icon_theme_get_default()
 
         try:
-            icon = icon_theme.load_icon(icon_name, icon_size, 0)
+            if build_icon_from_file:
+                icon = self._get_icon_from_image(icon_name,icon_size)
+            else:
+                icon = icon_theme.load_icon(icon_name, icon_size, 0)
         except:
             icon = icon_theme.load_icon(gtk.STOCK_DIALOG_QUESTION, icon_size, 0)
 
@@ -283,17 +380,60 @@ class EpisodeListModel(gtk.ListStore):
         return icon
 
 
+class PodcastChannelProxy(object):
+    def __init__(self, db, config, channels):
+        self._db = db
+        self._config = config
+        self.channels = channels
+        self.title =  _('All episodes')
+        self.description = _('from all podcasts')
+        self.parse_error = ''
+        self.url = ''
+        self.id = None
+        self._save_dir_size_set = False
+        self.save_dir_size = 0L
+        self.icon = None
+
+    def __getattribute__(self, name):
+        try:
+            return object.__getattribute__(self, name)
+        except AttributeError:
+            log('Unsupported method call (%s)', name, sender=self)
+
+    def get_statistics(self):
+        # Get the total statistics for all channels from the database
+        return self._db.get_total_count()
+
+    def get_all_episodes(self):
+        """Returns a generator that yields every episode"""
+        for channel in self.channels:
+            for episode in channel.get_all_episodes():
+                yield episode
+
+    def request_save_dir_size(self):
+        if not self._save_dir_size_set:
+            self.update_save_dir_size()
+        self._save_dir_size_set = True
+
+    def update_save_dir_size(self):
+        self.save_dir_size = util.calculate_size(self._config.download_dir)
+
+
 class PodcastListModel(gtk.ListStore):
     C_URL, C_TITLE, C_DESCRIPTION, C_PILL, C_CHANNEL, \
             C_COVER, C_ERROR, C_PILL_VISIBLE, \
             C_VIEW_SHOW_UNDELETED, C_VIEW_SHOW_DOWNLOADED, \
-            C_VIEW_SHOW_UNPLAYED, C_HAS_EPISODES = range(12)
+            C_VIEW_SHOW_UNPLAYED, C_HAS_EPISODES, C_SEPARATOR = range(13)
 
     SEARCH_COLUMNS = (C_TITLE, C_DESCRIPTION)
 
+    @classmethod
+    def row_separator_func(cls, model, iter):
+        return model.get_value(iter, cls.C_SEPARATOR)
+
     def __init__(self, max_image_side, cover_downloader):
         gtk.ListStore.__init__(self, str, str, str, gtk.gdk.Pixbuf, \
-                object, gtk.gdk.Pixbuf, str, bool, bool, bool, bool, bool)
+                object, gtk.gdk.Pixbuf, str, bool, bool, bool, bool, bool, bool)
 
         # Filter to allow hiding some episodes
         self._filter = self.filter_new()
@@ -308,13 +448,14 @@ class PodcastListModel(gtk.ListStore):
             self._max_image_side = max_image_side
         self._cover_downloader = cover_downloader
 
-
     def _filter_visible_func(self, model, iter):
         # If searching is active, set visibility based on search text
         if self._search_term is not None:
             key = self._search_term.lower()
             return any((key in model.get_value(iter, column).lower()) for column in self.SEARCH_COLUMNS)
 
+        if model.get_value(iter, self.C_SEPARATOR):
+            return True
         if self._view_mode == EpisodeListModel.VIEW_ALL:
             return model.get_value(iter, self.C_HAS_EPISODES)
         elif self._view_mode == EpisodeListModel.VIEW_UNDELETED:
@@ -356,6 +497,11 @@ class PodcastListModel(gtk.ListStore):
     def get_search_term(self):
         return self._search_term
 
+    def enable_separators(self, channeltree):
+        channeltree.set_row_separator_func(self._show_row_separator)
+
+    def _show_row_separator(self, model, iter):
+        return model.get_value(iter, self.C_SEPARATOR)
 
     def _resize_pixbuf_keep_ratio(self, url, pixbuf):
         """
@@ -408,16 +554,15 @@ class PodcastListModel(gtk.ListStore):
         else:
             return None
 
-    def _format_description(self, channel, count_new):
+    def _format_description(self, channel, total, deleted, \
+            new, downloaded, unplayed):
         title_markup = xml.sax.saxutils.escape(channel.title)
-        if gpodder.ui.fremantle:
-            return title_markup
         description_markup = xml.sax.saxutils.escape(util.get_first_line(channel.description) or ' ')
         d = []
-        if count_new:
+        if new:
             d.append('<span weight="bold">')
         d.append(title_markup)
-        if count_new:
+        if new:
             d.append('</span>')
         return ''.join(d+['\n', '<small>', description_markup, '</small>'])
 
@@ -427,15 +572,30 @@ class PodcastListModel(gtk.ListStore):
         else:
             return None
 
-    def set_channels(self, channels):
+    def set_channels(self, db, config, channels):
         # Clear the model and update the list of podcasts
         self.clear()
+
+        if config.podcast_list_view_all:
+            all_episodes = PodcastChannelProxy(db, config, channels)
+            iter = self.append()
+            self.set(iter, \
+                    self.C_URL, all_episodes.url, \
+                    self.C_CHANNEL, all_episodes, \
+                    self.C_COVER, all_episodes.icon, \
+                    self.C_SEPARATOR, False)
+            self.update_by_iter(iter)
+
+            iter = self.append()
+            self.set(iter, self.C_SEPARATOR, True)
+
         for channel in channels:
             iter = self.append()
             self.set(iter, \
                     self.C_URL, channel.url, \
                     self.C_CHANNEL, channel, \
-                    self.C_COVER, self._get_cover_image(channel))
+                    self.C_COVER, self._get_cover_image(channel), \
+                    self.C_SEPARATOR, False)
             self.update_by_iter(iter)
 
     def get_filter_path_from_url(self, url):
@@ -472,12 +632,16 @@ class PodcastListModel(gtk.ListStore):
     def update_by_iter(self, iter):
         # Given a GtkTreeIter, update volatile information
         channel = self.get_value(iter, self.C_CHANNEL)
+        if channel is None:
+            return
         total, deleted, new, downloaded, unplayed = channel.get_statistics()
+        description = self._format_description(channel, total, deleted, new, \
+                downloaded, unplayed)
 
         pill_image = self._get_pill_image(channel, downloaded, unplayed)
         self.set(iter, \
                 self.C_TITLE, channel.title, \
-                self.C_DESCRIPTION, self._format_description(channel, new), \
+                self.C_DESCRIPTION, description, \
                 self.C_ERROR, self._format_error(channel), \
                 self.C_PILL, pill_image, \
                 self.C_PILL_VISIBLE, pill_image != None, \
