@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -198,9 +198,14 @@ class PodcastChannel(PodcastModelObject):
             self.pubDate = time.time()
 
         if hasattr(feed.feed, 'image'):
-            if hasattr(feed.feed.image, 'href') and feed.feed.image.href:
-                old = self.image
-                self.image = feed.feed.image.href
+            for attribute in ('href', 'url'):
+                new_value = getattr(feed.feed.image, attribute, None)
+                if new_value is not None:
+                    log('Found cover art in %s: %s', attribute, new_value)
+                    self.image = new_value
+
+        if hasattr(feed.feed, 'icon'):
+            self.image = feed.feed.icon
 
         self.save()
 
@@ -594,12 +599,16 @@ class PodcastChannel(PodcastModelObject):
     
     @property
     def cover_file(self):
-        old_cover = os.path.join(self.save_dir, 'cover')
-        cover = os.path.join(self.save_dir, '.cover')
-        if os.path.exists(old_cover):
-            return old_cover
-        else:
-            return cover
+        new_name = os.path.join(self.save_dir, 'folder.jpg')
+        if not os.path.exists(new_name):
+            old_names = ('cover', '.cover')
+            for old_name in old_names:
+                filename = os.path.join(self.save_dir, old_name)
+                if os.path.exists(filename):
+                    shutil.move(filename, new_name)
+                    return new_name
+
+        return new_name
 
     def delete_episode_by_url(self, url):
         episode = self.db.load_episode(url, factory=self.episode_factory)
@@ -633,6 +642,9 @@ class PodcastEpisode(PodcastModelObject):
 
         return self
 
+    def has_website_link(self):
+        return bool(self.link) and (self.link != self.url)
+
     @staticmethod
     def from_feedparser_entry( entry, channel):
         episode = PodcastEpisode( channel)
@@ -660,7 +672,7 @@ class PodcastEpisode(PodcastModelObject):
             enclosure = entry.enclosures[0]
             if len(entry.enclosures) > 1:
                 for e in entry.enclosures:
-                    if hasattr( e, 'href') and hasattr( e, 'length') and hasattr( e, 'type') and (e.type.startswith('audio/') or e.type.startswith('video/')):
+                    if hasattr( e, 'href') and hasattr( e, 'length') and hasattr( e, 'type') and (e.type.startswith('audio/') or e.type.startswith('video/') or e.type.startswith('image/')):
                         if util.normalize_feed_url(e.href) is not None:
                             log( 'Selected enclosure: %s', e.href, sender = episode)
                             enclosure = e
@@ -669,7 +681,7 @@ class PodcastEpisode(PodcastModelObject):
         elif hasattr(entry, 'media_content'):
             media = getattr(entry, 'media_content')
             for m in media:
-                if 'url' in m and 'type' in m and (m['type'].startswith('audio/') or m['type'].startswith('video/')):
+                if 'url' in m and 'type' in m and (m['type'].startswith('audio/') or m['type'].startswith('video/') or m['type'].startswith('image/')):
                     if util.normalize_feed_url(m['url']) is not None:
                         log('Selected media_content: %s', m['url'], sender = episode)
                         episode.url=util.normalize_feed_url(m['url'])
@@ -790,13 +802,6 @@ class PodcastEpisode(PodcastModelObject):
             self.save()
             self.db.commit()
 
-    def format_episode_row_markup(self, include_description=False):
-        if include_description:
-            return '%s\n<small>%s</small>' % (xml.sax.saxutils.escape(self.title),
-                    xml.sax.saxutils.escape(self.one_line_description()))
-        else:
-            return xml.sax.saxutils.escape(self.title)
-
     @property
     def title_markup(self):
         return '%s\n<small>%s</small>' % (xml.sax.saxutils.escape(self.title),
