@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,16 +38,14 @@ from gpodder.liblogger import log
 
 from gpodder import util
 
-import gtk
-import gobject
-
 import xml.dom.minidom
-import xml.sax.saxutils
 
 import urllib
 import urllib2
 import os.path
 import os
+import platform
+import shutil
 
 from email.Utils import formatdate
 import gpodder
@@ -63,11 +61,7 @@ class Importer(object):
     contains workarounds to support odeo.com feeds.
     """
 
-    VALID_TYPES = ( 'rss', 'link' )
-
-    def read_url( self, url):
-        request = urllib2.Request( url, headers = {'User-agent': gpodder.user_agent})
-        return urllib2.urlopen( request).read()
+    VALID_TYPES = ('rss', 'link')
 
     def __init__( self, url):
         """
@@ -76,18 +70,18 @@ class Importer(object):
         """
         self.items = []
         try:
-            if url.startswith('/'):
-                # assume local filename
-                if os.path.exists(url):
-                    doc = xml.dom.minidom.parse( url)
-                else:
-                    log('Empty/non-existing OPML file', sender=self)
-                    return
+            if os.path.exists(url):
+                doc = xml.dom.minidom.parse(url)
             else:
-                doc = xml.dom.minidom.parseString( self.read_url( url))
+                doc = xml.dom.minidom.parseString(util.urlopen(url).read())
 
             for outline in doc.getElementsByTagName('outline'):
-                if outline.getAttribute('type') in self.VALID_TYPES and outline.getAttribute('xmlUrl') or outline.getAttribute('url'):
+                # Make sure we are dealing with a valid link type (ignore case)
+                otl_type = outline.getAttribute('type')
+                if otl_type is None or otl_type.lower() not in self.VALID_TYPES:
+                    continue
+
+                if outline.getAttribute('xmlUrl') or outline.getAttribute('url'):
                     channel = {
                         'url': outline.getAttribute('xmlUrl') or outline.getAttribute('url'),
                         'title': outline.getAttribute('title') or outline.getAttribute('text') or outline.getAttribute('xmlUrl') or outline.getAttribute('url'),
@@ -106,32 +100,6 @@ class Importer(object):
         except:
             log( 'Cannot import OPML from URL: %s', url, traceback=True, sender = self)
 
-    def format_channel( self, channel):
-        """
-        Formats a channel dictionary (as populated by the 
-        constructor) into a Pango markup string, suitable 
-        for output in GTK widgets.
-
-        The resulting string contains the title and description.
-        """
-        return '<b>%s</b>\n<span size="small">%s</span>' % ( xml.sax.saxutils.escape( urllib.unquote_plus( channel['title'])), xml.sax.saxutils.escape( channel['description']), )
-
-    def get_model( self):
-        """
-        Returns a gtk.ListStore with three columns:
-
-         - a bool that is initally set to False
-         - a descriptive Pango markup string created
-           by calling self.format_channel()
-         - the URL of the channel as string
-        """
-        model = gtk.ListStore( gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, gobject.TYPE_STRING)
-
-        for channel in self.items:
-            model.append( [ False, self.format_channel( channel), channel['url'] ])
-
-        return model
-
 
 
 class Exporter(object):
@@ -206,15 +174,21 @@ class Exporter(object):
             # try to save the new file, but keep the old one so we
             # don't end up with a clobbed, empty opml file.
             FREE_DISK_SPACE_AFTER = 1024*512
-            if util.get_free_disk_space(os.path.dirname(self.filename)) < 2*len(data)+FREE_DISK_SPACE_AFTER:
+            available = util.get_free_disk_space(os.path.dirname(self.filename))
+            if available < 2*len(data)+FREE_DISK_SPACE_AFTER and not gpodder.win32:
+                # FIXME: get_free_disk_space still unimplemented for win32
                 log('Not enough free disk space to save channel list to %s', self.filename, sender = self)
                 return False
             fp = open(self.filename+'.tmp', 'w')
             fp.write(data)
             fp.close()
-            os.rename(self.filename+'.tmp', self.filename)
+            if gpodder.win32:
+                # Win32 does not support atomic rename with os.rename
+                shutil.move(self.filename+'.tmp', self.filename)
+            else:
+                os.rename(self.filename+'.tmp', self.filename)
         except:
-            log( 'Could not open file for writing: %s', self.filename, sender = self)
+            log('Could not open file for writing: %s', self.filename, sender=self, traceback=True)
             return False
 
         return True
