@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,19 +22,19 @@
 # Thomas Perl <thp@perli.net> 2007-12-06
 # based on libipodsync.py (2006-04-05 Thomas Perl)
 
+import gpodder
 
 from gpodder import util
 from gpodder import services
 from gpodder import libconverter
-from gpodder import libtagupdate
 
 from gpodder.liblogger import log
-from gpodder.libgpodder import gl
-from gpodder.dbsqlite import db
 
 import time
 import calendar
 
+_ = gpodder.gettext
+
 gpod_available = True
 try:
     import gpod
@@ -82,14 +82,14 @@ import email.Utils
 import re
 
 
-def open_device():
-    device_type = gl.config.device_type
+def open_device(config):
+    device_type = config.device_type
     if device_type == 'ipod':
-        return iPodDevice()
+        return iPodDevice(config)
     elif device_type == 'filesystem':
-        return MP3PlayerDevice()
+        return MP3PlayerDevice(config)
     elif device_type == 'mtp':
-        return MTPDevice()
+        return MTPDevice(config)
     else:
         return None
 
@@ -139,7 +139,7 @@ class SyncTrack(object):
     def __init__(self, title, length, modified, **kwargs):
         self.title = title
         self.length = length
-        self.filesize = util.format_filesize(length, gl.config.use_si_units)
+        self.filesize = util.format_filesize(length)
         self.modified = modified
 
         # Set some (possible) keyword arguments to default values
@@ -152,7 +152,8 @@ class SyncTrack(object):
 
 
 class Device(services.ObservableService):
-    def __init__(self):
+    def __init__(self, config):
+        self._config = config
         self.cancelled = False
         self.allowed_types = ['audio', 'video']
         self.errors = []
@@ -169,7 +170,7 @@ class Device(services.ObservableService):
 
     def close(self):
         self.notify('status', _('Writing data to disk'))
-        if gl.config.sync_disks_after_transfer:
+        if self._config.sync_disks_after_transfer:
             successful_sync = (os.system('sync') == 0)
         else:
             log('Not syncing disks. Unmount your device before unplugging.', sender=self)
@@ -179,28 +180,31 @@ class Device(services.ObservableService):
         return True
 
     def add_tracks(self, tracklist=[], force_played=False):
-        for id, track in enumerate(tracklist):
+        for track in list(tracklist):
+            # Filter tracks that are not meant to be synchronized
+            does_not_exist = not track.was_downloaded(and_exists=True)
+            exclude_played = track.is_played and not force_played and \
+                    self._config.only_sync_not_played
+            wrong_type = track.file_type() not in self.allowed_types
+
+            if does_not_exist or exclude_played or wrong_type:
+                log('Excluding %s from sync', track.title, sender=self)
+                tracklist.remove(track)
+
+        compare_episodes = lambda a, b: cmp(a.pubDate, b.pubDate)
+        for id, track in enumerate(sorted(tracklist, cmp=compare_episodes)):
             if self.cancelled:
                 return False
 
             self.notify('progress', id+1, len(tracklist))
 
-            if not track.was_downloaded(and_exists=True):
-                continue
-
-            if track.is_played and gl.config.only_sync_not_played and not force_played:
-                continue
-
-            if track.file_type() not in self.allowed_types:
-                continue
-
             added = self.add_track(track)
 
-            if gl.config.on_sync_mark_played:
+            if self._config.on_sync_mark_played:
                 log('Marking as played on transfer: %s', track.url, sender=self)
-                db.mark_episode(track.url, is_played=True)
+                track.mark(is_played=True)
 
-            if added and gl.config.on_sync_delete:
+            if added and self._config.on_sync_delete and not track.is_locked:
                 log('Removing episode after transfer: %s', track.url, sender=self)
                 track.delete_from_disk()
         return True
@@ -212,7 +216,7 @@ class Device(services.ObservableService):
         assert filename is not None
         (fn, extension) = os.path.splitext(filename)
         if libconverter.converters.has_converter(extension):
-            if gl.config.disable_pre_sync_conversion:
+            if self._config.disable_pre_sync_conversion:
                 log('Pre-sync conversion is not enabled, set disable_pre_sync_conversion to "False" to enable')
                 return filename
 
@@ -224,9 +228,6 @@ class Device(services.ObservableService):
                 log('Cannot convert %s', original_filename, sender=self)
                 return filename
 
-            if not libtagupdate.update_metadata_on_file(local_filename, title=episode.title, artist=episode.channel.title):
-                log('Could not set metadata on converted file %s', local_filename, sender=self)
-
             return str(local_filename)
 
         return filename
@@ -254,17 +255,18 @@ class Device(services.ObservableService):
     def episode_on_device(self, episode):
         return self._track_on_device(episode.title)
 
-    def _track_on_device( self, track_name ):
+    def _track_on_device(self, track_name):
         for t in self.tracks_list:
-            if track_name == t.title:
+            title = t.title
+            if track_name == title:
                 return t
-        return False
+        return None
 
 class iPodDevice(Device):
-    def __init__(self):
-        Device.__init__(self)
+    def __init__(self, config):
+        Device.__init__(self, config)
 
-        self.mountpoint = str(gl.config.ipod_mount)
+        self.mountpoint = str(self._config.ipod_mount)
 
         self.itdb = None
         self.podcast_playlist = None
@@ -303,35 +305,33 @@ class iPodDevice(Device):
             self.notify('status', _('Saving iPod database'))
             gpod.itdb_write(self.itdb, None)
             self.itdb = None
-
-            if gl.config.ipod_write_gtkpod_extended:
-                # Fix up iTunesDB.ext (gtkpod extended database),
-                # so gtkpod will not complain about a wrong sha1sum
-                self.notify('status', _('Writing extended gtkpod database'))
-                ext_filename = os.path.join(self.mountpoint, 'iPod_Control', 'iTunes', 'iTunesDB.ext')
-                idb_filename = os.path.join(self.mountpoint, 'iPod_Control', 'iTunes', 'iTunesDB')
-                if os.path.exists(ext_filename) and os.path.exists(idb_filename):
-                    try:
-                        db = gpod.ipod.Database(self.mountpoint)
-                        gpod.gtkpod.parse(ext_filename, db, idb_filename)
-                        gpod.gtkpod.write(ext_filename, db, idb_filename)
-                        db.close()
-                    except:
-                        log('Error when writing iTunesDB.ext', sender=self, traceback=True)
-                else:
-                    log('I could not find %s or %s. Will not update extended gtkpod DB.', ext_filename, idb_filename, sender=self)
-            else:
-                log('Not writing extended gtkpod DB. Set "ipod_write_gpod_extended" to True if I should write it.', sender=self)
             
         Device.close(self)
         return True
 
+    def update_played_or_delete(self, channel, episodes, delete_from_db):
+        """
+        Check whether episodes on ipod are played and update as played
+        and delete if required.
+        """
+        for episode in episodes:
+            track = self.episode_on_device(episode)
+            if track:
+                gtrack = track.libgpodtrack
+                if gtrack.playcount > 0:
+                    if delete_from_db and not gtrack.rating:
+                        log('Deleting episode from db %s', gtrack.title, sender=self)
+                        channel.delete_episode_by_url(gtrack.podcasturl)
+                    else:
+                        log('Marking episode as played %s', gtrack.title, sender=self)
+                        episode.mark(is_played=True)
+
     def purge(self):
         for track in gpod.sw_get_playlist_tracks(self.podcasts_playlist):
             if gpod.itdb_filename_on_ipod(track) is None:
                 log('Episode has no file: %s', track.title, sender=self)
                 # self.remove_track_gpod(track)
-            elif track.mark_unplayed == 1 and not track.rating:
+            elif track.playcount > 0  and not track.rating:
                 log('Purging episode: %s', track.title, sender=self)
                 self.remove_track_gpod(track)
 
@@ -341,12 +341,12 @@ class iPodDevice(Device):
             filename = gpod.itdb_filename_on_ipod(track)
             length = util.calculate_size(filename)
 
-            age_in_days = util.file_age_in_days(filename)
-            modified = util.file_age_to_string(age_in_days)
+            timestamp = util.file_modification_timestamp(filename)
+            modified = util.format_date(timestamp)
             released = gpod.itdb_time_mac_to_host(track.time_released)
             released = util.format_date(released)
 
-            t = SyncTrack(track.title, length, modified, libgpodtrack=track, playcount=track.playcount, released=released, podcast=track.artist)
+            t = SyncTrack(track.title, length, modified, modified_sort=timestamp, libgpodtrack=track, playcount=track.playcount, released=released, podcast=track.artist)
             tracks.append(t)
         return tracks        
 
@@ -370,9 +370,9 @@ class iPodDevice(Device):
         for track in gpod.sw_get_playlist_tracks(self.podcasts_playlist):
             if episode.url == track.podcasturl:
                 if track.playcount > 0:
-                    db.mark_episode(track.podcasturl, is_played=True)
+                    episode.mark(is_played=True)
                 # Mark as played on iPod if played locally (and set podcast flags)
-                self.set_podcast_flags(track)
+                self.set_podcast_flags(track, episode)
                 return True
 
         original_filename = episode.local_filename(create=False)
@@ -383,7 +383,9 @@ class iPodDevice(Device):
 
         if util.calculate_size(original_filename) > self.get_free_space():
             log('Not enough space on %s, sync aborted...', self.mountpoint, sender = self)
-            self.errors.append( _('Error copying %s: Not enough free disk space on %s') % (episode.title, self.mountpoint))
+            d = {'episode': episode.title, 'mountpoint': self.mountpoint}
+            message =_('Error copying %(episode)s: Not enough free space on %(mountpoint)s')
+            self.errors.append(message % d)
             self.cancelled = True
             return False
 
@@ -426,7 +428,7 @@ class iPodDevice(Device):
             track.filetype = 'm4v'
             track.mediatype = 0x00000006
 
-        self.set_podcast_flags(track)
+        self.set_podcast_flags(track, episode)
         self.set_cover_art(track, local_filename)
 
         gpod.itdb_track_add(self.itdb, track, -1)
@@ -439,16 +441,20 @@ class iPodDevice(Device):
 
         return True
 
-    def set_podcast_flags(self, track):
+    def set_podcast_flags(self, track, episode):
         try:
             # Set blue bullet for unplayed tracks on 5G iPods
-            episode = db.load_episode(track.podcasturl)
-            if episode['is_played']:
+            if episode.is_played:
                 track.mark_unplayed = 0x01
                 if track.playcount == 0:
                     track.playcount = 1
             else:
-                track.mark_unplayed = 0x02
+                if track.playcount > 0 or track.bookmark_time > 0:
+                    #track is partially played so no blue bullet
+                    track.mark_unplayed = 0x01
+                else:
+                    #totally unplayed
+                    track.mark_unplayed = 0x02
 
             # Set several flags for to podcast values
             track.remember_playback_position = 0x01
@@ -481,7 +487,8 @@ class iPodDevice(Device):
             log('Error getting cover using eyeD3', sender=self)
 
         try:
-            cover_filename = os.path.join(os.path.dirname(local_filename), 'cover')
+            cover_filename = os.path.join(os.path.dirname(local_filename), 'folder.jpg')
+
             if os.path.isfile(cover_filename):
                 gpod.itdb_track_set_thumbnails(track, cover_filename)
                 return True
@@ -496,14 +503,9 @@ class MP3PlayerDevice(Device):
     # .scrobbler.log, add them to this list
     scrobbler_log_filenames = ['.scrobbler.log']
 
-    # This is the maximum length of a file name that is
-    # created on the MP3 player, because FAT32 has a
-    # 255-character limit for the whole path
-    MAX_FILENAME_LENGTH = gl.config.mp3_player_max_filename_length
-
-    def __init__(self):
-        Device.__init__(self)
-        self.destination = gl.config.mp3_player_folder
+    def __init__(self, config):
+        Device.__init__(self, config)
+        self.destination = self._config.mp3_player_folder
         self.buffer_size = 1024*1024 # 1 MiB
         self.scrobbler_log = []
 
@@ -517,7 +519,7 @@ class MP3PlayerDevice(Device):
             self.notify('status', _('MP3 player opened'))
             # build the initial tracks_list
             self.tracks_list = self.get_all_tracks()
-            if gl.config.mp3_player_use_scrobbler_log:
+            if self._config.mp3_player_use_scrobbler_log:
                 mp3_player_mount_point = util.find_mount_point(self.destination)
                 # If a moint point cannot be found look inside self.destination for scrobbler_log_filenames
                 # this prevents us from os.walk()'ing the entire / filesystem
@@ -531,19 +533,19 @@ class MP3PlayerDevice(Device):
             return False
 
     def add_track(self, episode):
-        self.notify('status', _('Adding %s') % episode.title)
+        self.notify('status', _('Adding %s') % episode.title.decode('utf-8', 'ignore'))
 
-        if gl.config.fssync_channel_subfolders:
+        if self._config.fssync_channel_subfolders:
             # Add channel title as subfolder
             folder = episode.channel.title
             # Clean up the folder name for use on limited devices
-            folder = util.sanitize_filename(folder, self.MAX_FILENAME_LENGTH)
+            folder = util.sanitize_filename(folder, self._config.mp3_player_max_filename_length)
             folder = os.path.join(self.destination, folder)
         else:
             folder = self.destination
 
         from_file = util.sanitize_encoding(self.convert_track(episode))
-        filename_base = util.sanitize_filename(episode.sync_filename(), self.MAX_FILENAME_LENGTH)
+        filename_base = util.sanitize_filename(episode.sync_filename(self._config.custom_sync_name_enabled, self._config.custom_sync_name), self._config.mp3_player_max_filename_length)
 
         to_file = filename_base + os.path.splitext(from_file)[1].lower()
 
@@ -562,25 +564,27 @@ class MP3PlayerDevice(Device):
                 log('Cannot create folder on MP3 player: %s', folder, sender=self)
                 return False
 
-        if (gl.config.mp3_player_use_scrobbler_log and not episode.is_played
-                and [episode.channel.title, episode.title] in self.scrobbler_log):
-            log('Marking "%s" from "%s" as played', episode.title, episode.channel.title, sender=self)
-            db.mark_episode(episode.url, is_played=True)
+        if self._config.mp3_player_use_scrobbler_log and not episode.is_played:
+            # FIXME: This misses some things when channel.title<>album tag which is what
+            # the scrobbling entity will be using.
+            if [episode.channel.title, episode.title] in self.scrobbler_log:
+                log('Marking "%s" from "%s" as played', episode.title, episode.channel.title, sender=self)
+                episode.mark(is_played=True)
 
-        if gl.config.rockbox_copy_coverart and not os.path.exists(os.path.join(folder, 'cover.bmp')):
+        if self._config.rockbox_copy_coverart and not os.path.exists(os.path.join(folder, 'cover.bmp')):
             log('Creating Rockbox album art for "%s"', episode.channel.title, sender=self)
             self.copy_player_cover_art(folder, from_file, \
-            'cover.bmp', 'BMP', gl.config.rockbox_coverart_size)
+            'cover.bmp', 'BMP', self._config.rockbox_coverart_size)
 
-        if gl.config.custom_player_copy_coverart \
+        if self._config.custom_player_copy_coverart \
         and not os.path.exists(os.path.join(folder, \
-        gl.config.custom_player_coverart_name)):
+        self._config.custom_player_coverart_name)):
             log('Creating custom player album art for "%s"',
                 episode.channel.title, sender=self)
             self.copy_player_cover_art(folder, from_file, \
-            gl.config.custom_player_coverart_name, \
-            gl.config.custom_player_coverart_format, \
-            gl.config.custom_player_coverart_size)
+            self._config.custom_player_coverart_name, \
+            self._config.custom_player_coverart_format, \
+            self._config.custom_player_coverart_size)
 
         if not os.path.exists(to_file):
             log('Copying %s => %s', os.path.basename(from_file), to_file.decode(util.encoding), sender=self)
@@ -592,14 +596,16 @@ class MP3PlayerDevice(Device):
         try:
             out_file = open(to_file, 'wb')
         except IOError, ioerror:
-            self.errors.append(_('Error opening %s: %s') % (ioerror.filename, ioerror.strerror))
+            d = {'filename': ioerror.filename, 'message': ioerror.strerror}
+            self.errors.append(_('Error opening %(filename)s: %(message)s') % d)
             self.cancel()
             return False
 
         try:
             in_file = open(from_file, 'rb')
         except IOError, ioerror:
-            self.errors.append(_('Error opening %s: %s') % (ioerror.filename, ioerror.strerror))
+            d = {'filename': ioerror.filename, 'message': ioerror.strerror}
+            self.errors.append(_('Error opening %(filename)s: %(message)s') % d)
             self.cancel()
             return False
 
@@ -637,7 +643,7 @@ class MP3PlayerDevice(Device):
     def get_all_tracks(self):
         tracks = []
 
-        if gl.config.fssync_channel_subfolders:
+        if self._config.fssync_channel_subfolders:
             files = glob.glob(os.path.join(self.destination, '*', '*'))
         else:
             files = glob.glob(os.path.join(self.destination, '*'))
@@ -645,27 +651,27 @@ class MP3PlayerDevice(Device):
         for filename in files:
             (title, extension) = os.path.splitext(os.path.basename(filename))
             length = util.calculate_size(filename)
-         
-            age_in_days = util.file_age_in_days(filename)
-            modified = util.file_age_to_string(age_in_days)
-            if gl.config.fssync_channel_subfolders:
+
+            timestamp = util.file_modification_timestamp(filename)
+            modified = util.format_date(timestamp)
+            if self._config.fssync_channel_subfolders:
                 podcast_name = os.path.basename(os.path.dirname(filename))
             else:
                 podcast_name = None
-         
-            t = SyncTrack(title, length, modified, filename=filename, podcast=podcast_name)
+
+            t = SyncTrack(title, length, modified, modified_sort=timestamp, filename=filename, podcast=podcast_name)
             tracks.append(t)
         return tracks
 
     def episode_on_device(self, episode):
-        e = util.sanitize_filename(episode.sync_filename(), gl.config.mp3_player_max_filename_length)
+        e = util.sanitize_filename(episode.sync_filename(self._config.custom_sync_name_enabled, self._config.custom_sync_name), self._config.mp3_player_max_filename_length)
         return self._track_on_device(e)
 
     def remove_track(self, track):
         self.notify('status', _('Removing %s') % track.title)
         util.delete_file(track.filename)
         directory = os.path.dirname(track.filename)
-        if self.directory_is_empty(directory) and gl.config.fssync_channel_subfolders:
+        if self.directory_is_empty(directory) and self._config.fssync_channel_subfolders:
             try:
                 os.rmdir(directory)
             except:
@@ -700,6 +706,8 @@ class MP3PlayerDevice(Device):
         """
         try:
             cover_loc = os.path.join(os.path.dirname(local_filename), 'cover')
+            if not os.path.exists(cover_loc):
+                cover_loc = os.path.join(os.path.dirname(local_filename), '.cover')
             cover_dst = os.path.join(destination, cover_dst_name)
             if os.path.isfile(cover_loc):
                 log('Creating cover art file on player', sender=self)
@@ -734,22 +742,32 @@ class MP3PlayerDevice(Device):
             return False
 
         try:
-            # regex that can be used to get all the data from a scrobbler.log entry
-            entry_re = re.compile('^(.*)\t(.*)\t(.*)\t(.*)\t(.*)\t(.*)\t(.*)$')
+            # Scrobble Log Format: http://www.audioscrobbler.net/wiki/Portable_Player_Logging
+            # Notably some fields are optional so will appear as \t\t.
+            # Conforming scrobblers should strip any \t's from the actual fields.
             for entry in entries:
-                match_obj = re.match(entry_re, entry)
-                # L means at least 50% of the track was listened to (S means < 50%)
-                if match_obj and match_obj.group(6).strip().lower() == 'l':
-                    # append [artist_name, track_name]
-                    self.scrobbler_log.append([match_obj.group(1), match_obj.group(3)])
+                entry = entry.split('\t')
+                if len(entry)>=5:
+                    artist, album, track, pos, length, rating = entry[:6]
+                    # L means at least 50% of the track was listened to (S means < 50%)
+                    if 'L' in rating:
+                        # Whatever is writing the logs will only have the taginfo in the
+                        # file to work from. Mostly album~=channel name
+                        if len(track):
+                            self.scrobbler_log.append([album, track])
+                        else:
+                            log('Skipping logging of %s (missing track)', album)
+                else:
+                    log('Skipping scrobbler entry: %d elements %s', len(entry), entry)
+                    
         except:
             log('Error while parsing "%s".', log_file, sender=self)
 
         return True
 
 class MTPDevice(Device):
-    def __init__(self):
-        Device.__init__(self)
+    def __init__(self, config):
+        Device.__init__(self, config)
         self.__model_name = None
         self.__MTPDevice = pymtp.MTP()
 
@@ -890,7 +908,9 @@ class MTPDevice(Device):
             metadata.duration = get_track_length(str(filename))
 
             # send the file
-            self.__MTPDevice.send_track_from_file( filename, episode.basename, metadata, 0, callback=self.__callback)
+            self.__MTPDevice.send_track_from_file(filename,
+                    episode.basename + episode.extension(),
+                    metadata, 0, callback=self.__callback)
         except:
             log('unable to add episode %s', episode.title, sender=self, traceback=True)
             return False
@@ -926,10 +946,12 @@ class MTPDevice(Device):
             date = self.__mtp_to_date(track.date)
             if not date:
                 modified = track.date # not a valid mtp date. Display what mtp gave anyway
+                modified_sort = -1 # no idea how to sort invalid date
             else:
                 modified = util.format_date(date)
+                modified_sort = date
 
-            t = SyncTrack(title, length, modified, mtptrack=track, podcast=artist)
+            t = SyncTrack(title, length, modified, modified_sort=modified_sort, mtptrack=track, podcast=artist)
             tracks.append(t)
         return tracks
 
