@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ import platform
 import glob
 import stat
 import shlex
+import socket
 
 import re
 import subprocess
@@ -61,6 +62,7 @@ import StringIO
 import xml.dom.minidom
 
 _ = gpodder.gettext
+N_ = gpodder.ngettext
 
 
 # Try to detect OS encoding (by Leonid Ponomarev)
@@ -126,7 +128,7 @@ def normalize_feed_url(url):
     and lazy typists (see the source for details).
 
     >>> normalize_feed_url('fb:43FPodcast')
-    'http://feeds2.feedburner.com/43FPodcast'
+    'http://feeds.feedburner.com/43FPodcast'
     """
     if not url or len(url) < 8:
         return None
@@ -135,7 +137,7 @@ def normalize_feed_url(url):
     # keystrokes that you have to use.
     # Feel free to suggest other useful prefixes, and I'll add them here.
     PREFIXES = {
-            'fb:': 'http://feeds2.feedburner.com/%s',
+            'fb:': 'http://feeds.feedburner.com/%s',
             'yt:': 'http://www.youtube.com/rss/user/%s/videos.rss',
             'sc:': 'http://soundcloud.com/%s',
     }
@@ -334,16 +336,14 @@ def file_age_to_string(days):
     >>> file_age_to_string(0)
     ''
     >>> file_age_to_string(1)
-    u'one day ago'
+    u'1 day ago'
     >>> file_age_to_string(2)
     u'2 days ago'
     """
-    if days == 1:
-        return _('one day ago')
-    elif days > 1:
-        return _('%d days ago') % days
-    else:
+    if days < 1:
         return ''
+    else:
+        return N_('%d day ago', '%d days ago', days) % days
 
 
 def get_free_disk_space_win32(path):
@@ -654,7 +654,7 @@ def file_type_by_extension(extension):
 
     if type is not None and '/' in type:
         filetype, rest = type.split('/', 1)
-        if filetype in ('audio', 'video'):
+        if filetype in ('audio', 'video', 'image'):
             return filetype
     
     return None
@@ -1008,7 +1008,7 @@ def format_seconds_to_hour_min_sec(seconds):
     """
 
     if seconds < 1:
-        return _('0 seconds')
+        return N_('%d second', '%d seconds', seconds) % seconds
 
     result = []
 
@@ -1018,40 +1018,28 @@ def format_seconds_to_hour_min_sec(seconds):
     minutes = seconds/60
     seconds = seconds%60
 
-    if hours == 1:
-        result.append(_('1 hour'))
-    elif hours > 1:
-        result.append(_('%i hours') % hours)
+    if hours:
+        result.append(N_('%d hour', '%d hours', hours) % hours)
 
-    if minutes == 1:
-        result.append(_('1 minute'))
-    elif minutes > 1:
-        result.append(_('%i minutes') % minutes)
+    if minutes:
+        result.append(N_('%d minute', '%d minutes', minutes) % minutes)
 
-    if seconds == 1:
-        result.append(_('1 second'))
-    elif seconds > 1:
-        result.append(_('%i seconds') % seconds)
+    if seconds:
+        result.append(N_('%d second', '%d seconds', seconds) % seconds)
 
     if len(result) > 1:
         return (' '+_('and')+' ').join((', '.join(result[:-1]), result[-1]))
     else:
         return result[0]
 
-def proxy_request(url, proxy=None, method='HEAD'):
-    if proxy is None or proxy.strip() == '':
-        (scheme, netloc, path, parms, qry, fragid) = urlparse.urlparse(url)
-        conn = httplib.HTTPConnection(netloc)
-        start = len(scheme) + len('://') + len(netloc)
-        conn.request(method, url[start:])
-    else:
-        (scheme, netloc, path, parms, qry, fragid) = urlparse.urlparse(proxy)
-        conn = httplib.HTTPConnection(netloc)
-        conn.request(method, url)
-
+def http_request(url, method='HEAD'):
+    (scheme, netloc, path, parms, qry, fragid) = urlparse.urlparse(url)
+    conn = httplib.HTTPConnection(netloc)
+    start = len(scheme) + len('://') + len(netloc)
+    conn.request(method, url[start:])
     return conn.getresponse()
 
-def get_episode_info_from_url(url, proxy=None):
+def get_episode_info_from_url(url):
     """
     Try to get information about a podcast episode by sending
     a HEAD request to the HTTP server and parsing the result.
@@ -1062,16 +1050,13 @@ def get_episode_info_from_url(url, proxy=None):
       "length": The size of the file in bytes
       "pubdate": The unix timestamp for the pubdate
 
-    If the "proxy" parameter is used, it has to be the URL 
-    of the HTTP proxy server to use, e.g. http://proxy:8080/
-    
     If there is an error, this function returns {}. This will
     only function with http:// and https:// URLs.
     """
     if not (url.startswith('http://') or url.startswith('https://')):
         return {}
 
-    r = proxy_request(url, proxy)
+    r = http_request(url)
     result = {}
 
     log('Trying to get metainfo for %s', url)
@@ -1114,9 +1099,23 @@ def gui_open(filename):
             log('Using Nokia Media Player to open %s', filename)
             context = osso.Context('gPodder', gpodder.__version__, False)
             filename = filename.encode('utf-8')
+
+            # Fix for Maemo bug 7162 (for episodes with "#" in filename)
+            filename = 'file://' + urllib.quote(filename)
+
             rpc = osso.Rpc(context)
-            service, path = 'com.nokia.mediaplayer', '/com/nokia/mediaplayer'
-            rpc.rpc_run(service, path, service, 'mime_open', (filename,))
+            app = 'mediaplayer'
+
+            _unneeded, extension = os.path.splitext(filename.lower())
+
+            # Fix for Maemo bug 5588 (use PDF viewer and images app)
+            if extension == '.pdf':
+                app = 'osso_pdfviewer'
+            elif extension in ('.jpg', '.jpeg', '.png'):
+                app = 'image_viewer'
+
+            svc, path = (x % app for x in ('com.nokia.%s', '/com/nokia/%s'))
+            rpc.rpc_run(svc, path, svc, 'mime_open', (filename,))
         elif gpodder.win32:
             os.startfile(filename)
         else:
@@ -1359,3 +1358,17 @@ def run_external_command(command_line):
 
     threading.Thread(target=open_process, args=(command_line,)).start()
 
+def get_hostname():
+    """Return the hostname of this computer
+
+    This can be implemented in a different way on each
+    platform and should yield a unique-per-user device ID.
+    """
+    nodename = platform.node()
+
+    if nodename:
+        return nodename
+
+    # Fallback - but can this give us "localhost"?
+    return socket.gethostname()
+
